@@ -7,33 +7,58 @@ use Module::CoreList qw();
 use autodie;
 use Moose;
 use IPC::Cmd qw[can_run run];
+use Dependencies::Searcher::AckRequester;
+use Cwd;
 
 # These modules will be used throught a system call
 # Module::Version;
 # App::Ack;
 
-our $VERSION = '0.05_01';
+our $VERSION = '0.05_02';
 
+=head1 NAME
+
+Dependencies::Searcher - Search recursively dependencies used in a module's directory
+and build a report that can be used as a Carton cpanfile.
+
+=cut
+
+=head1 SYNOPSIS
+
+Maybe you don't want to have to list all the dependencies of your Perl application by
+hand and want an automated way to build it. Maybe you forgot to do it for a long time
+ago. During this time, you've add lots of CPAN modules. Carton is here to help you
+manage dependencies between your development environment and production, but how to
+keep track of the list of modules you will pass to to Carton?
+
+Event if it is a no brainer to keep track of this list, it can be much better not to
+have to do it.
+
+You will need a tool that will check for any 'requires' or 'use' in your module package,
+and report it into a file that could be used as a Carton cpanfile. Any duplicated entry
+will be removed and modules versions will be checked and made available. Core modules
+will be ommited because you don't need to install them.
+
+This project has begun because it happens to me, and I don't want to search for modules
+to install by hand, I just want to run a simple script that update the list in a simple
+way. It was much more longer to write the module than to search by hand but I wish it
+will be usefull for others.
+
+    use Dependencies::Searcher;
+
+    my $searcher = Dependencies::Searcher->new();
+    my @elements = $searcher->get_files();
+    my $path = $searcher->build_full_path(@elements);
+    my @uses = $searcher->get_modules($path, "use");
+    my @uniq_modules = $searcher->uniq(@uses);
+
+    $searcher->dissociate(@uniq_modules);
+
+    $searcher->generate_report($searcher->non_core_modules);
+
+=cut
 
 # Init parameters
-has 'use_pattern' => (
-    is       => 'ro',
-    isa      => 'Str',
-    required => 1,
-);
-
-has 'requires_pattern' => (
-    is       => 'ro',
-    isa      => 'Str',
-    required => 1,
-);
-
-has 'parameters' => (
-    is       => 'ro',
-    isa      => 'Str',
-    required => 1,
-);
-
 has 'non_core_modules' => (
     traits     => ['Array'],
     is         => 'rw',
@@ -56,21 +81,25 @@ has 'core_modules' => (
     },
 );
 
-
+#
+# BUGGED : TRY TO USE IPC::Cmd, changed API !!!
+#
 sub get_modules {
-    my ($self, $path, $flag) = @_;
+    my ($self, $pattern, @path) = @_;
 
-    my $request = "";
+    #p @path;
+    #p $pattern;
 
-    if ($flag eq "use") {
-	$request = $self->parameters . " " . $self->use_pattern . " " . $path;
-    } elsif ($flag eq "require") {
-	$request = $self->parameters . " " . $self->requires_pattern . " " . $path;
-    } else {
-	die "Pattern flag is required : use or require";
-    }
+    my $ack_requester = Dependencies::Searcher::AckRequester->new();
 
-    my @moduls = `ack $request`;
+    my @moduls;
+
+    my @params = ('--perl', '-hi', $pattern, @path);
+
+    my $requester = Dependencies::Searcher::AckRequester->new();
+    my $ack_path = $requester->get_path();
+    my $cmd_use = $requester->build_cmd(@params);
+    @moduls = $requester->ack($cmd_use);
 
     if ( defined $moduls[0]) {
 	if ($moduls[0] =~ m/^use/ or $moduls[0] =~ m/^require/) {
@@ -79,29 +108,38 @@ sub get_modules {
 	    die "Failed to retrieve modules with Ack";
 	}
     } else {
-	say "No $self->pattern found !";
+	say "No use or require found !";
     }
 }
 
 sub get_files {
     my $self = shift;
+    # This prefix will allow a more portable module
+    my $prefix = getcwd();
+
     my @structure;
     $structure[0] = "";
     $structure[1] = "";
     $structure[2] = "";
-    if (-d "lib") {
-	$structure[0] = "lib";
+    if (-d $prefix."/lib") {
+
+	$structure[0] = $prefix."/lib";
     } else {
+	# 
+	# TEST IF THE PATH IS OK ???
+	#
+	#
 	die "Don't look like we are working on a Perl module";
     }
 
-    if (-f "Makefile.PL") {
-	$structure[1] = "Makefile.PL";
+    if (-f $prefix."/Makefile.PL") {
+	$structure[1] = $prefix."/Makefile.PL";
     }
 
-    if (-d "script") {
-	$structure[2] = "script";
+    if (-d $prefix."/script") {
+	$structure[2] = $prefix."/script";
     }
+
     return @structure;
 }
 
@@ -209,6 +247,7 @@ sub dissociate {
 	    # push @{ $self->core_modules }, $nc_module;
 
 	    # The "Moose" trait way
+	    # https://metacpan.org/module/Moose::Meta::Attribute::Native::Trait::Array
 	    $self->add_core_module($nc_module);
 
 	} else {
@@ -243,55 +282,21 @@ sub generate_report {
     close $cpanfile_fh;
 }
 
-=head1 NAME
-
-Dependencies::Searcher - Search recursively dependencies used in a module's 
-directory and build a report that can be used as a Carton cpanfile.
-
-=cut
-
-=head1 SYNOPSIS
-
-Maybe you don't want to have to list all the dependencies of your application by hand,
-or maybe you forgot to do it for a long time ago. During this time, you've add lots of
-CPAN modules. Carton is here to help you manage dependencies between your development 
-environment and production, but how to keep track of the list of modules you will give
-to Carton?
-
-You will need a tool that will check for any 'requires' or 'use' in your module package, 
-and report it into a file that could be used as a Carton cpanfile. Any duplicated entry 
-will be removed and versions are available.
-
-This project has begun because it happens to me, and I don't want to search for modules
-to install, I just want to run a simple script that update the list in a simple way.
-
-    use Dependencies::Searcher;
-
-    my $searcher = Dependencies::Searcher->new();
-    my @elements = $searcher->get_files();
-    my $path = $searcher->build_full_path(@elements);
-    my @uses = $searcher->get_modules($path, "use");		
-    my @uniq_modules = $searcher->uniq(@uses);
-
-    $searcher->dissociate(@uniq_modules);
-
-    $searcher->generate_report($searcher->non_core_modules);
-
 =head1 SUBROUTINES/METHODS
 
 This is work in progress...
 
-=head2 get_modules
+=head2 Dependencies::Searcher->get_modules()
 
 Us-e Ack to get modules and store lines into arrays
 
 =cut
 
-=head2 get_files
+=head2 Dependencies::Searcher->get_files()
 
 =cut
 
-=head2 build_full_path
+=head2 Dependencies::Searcher->build_full_path()
 
 Retrieve names of :
  * lib/ directory, if it don't exist, we don't care and die
@@ -301,50 +306,42 @@ Retrieve names of :
 
 =cut
 
-=head2 merge_dependencies 
+=head2 Dependencies::Searcher->merge_dependencies()
 
 Merge use and requires
 
 =cut
 
-=head2 make_it_real
+=head2 Dependencies::Searcher->make_it_real()
 
 Remove special cases that can't be interesting.
 
 =cut
 
-=head2 clean_everything
+=head2 Dependencies::Searcher->clean_everything()
 
 Remove everything but the module name. Remove dirt, clean stuffs...
 
 =cut
 
-=head2 uniq
+=head2 Dependencies::Searcher->uniq()
 
 Make each array element uniq
 
 =cut
 
-=head2 dissociate
+=head2 Dependencies::Searcher->dissociate()
 
 Dissociate core / non-core modules
 
 =cut
 
-=head2 generate_report
+=head2 Dependencies::Searcher->generate_report()
 
 Generate the cpanfile for Carton, with optionnal version number
 
 =cut
 
-=head2
-
-=cut
-
-=head2
-
-=cut
-
 =head1 AUTHOR
 
 smonff, C<< <smonff at gmail.com> >>
